@@ -1,6 +1,8 @@
 from synoptic.datamodel import \
         Item, ItemVersion, Tag, ViewOrdering, ViewOrderingEntry, \
-        find_tags
+        store_itemversion,  \
+        get_current_itemversions_join, \
+        query_itemversions
 
 
 
@@ -38,34 +40,15 @@ def get_static_file(filename):
 
 
 
-def store_itemversion(dbsession, contents, tags, item_id=None):
-    if item_id is None:
-        item = Item()
-        dbsession.save(item)
-    else:
-        assert isinstance(item_id, int)
-        item = dbsession.query(Item).get(item_id)
-
-    from time import time
-    itemversion = ItemVersion(
-            item,
-            time(),
-            find_tags(dbsession, tags, create_them=True),
-            contents,
-            )
-    dbsession.save(itemversion)
-
-    return itemversion
-
-
-
-
 def import_file(dbsession, text):
     lines = text.split("\n")
 
     tags_label = "TAGS: "
     separator = 60*"-"
 
+    from time import time
+    timestamp = time()
+
     idx = 0
     while idx < len(lines):
         assert lines[idx].startswith(tags_label)
@@ -80,16 +63,17 @@ def import_file(dbsession, text):
 
         idx += 1  # skip separator
 
-        store_itemversion(dbsession, "\n".join(body), tags)
+        store_itemversion(dbsession, "\n".join(body), tags, timestamp=timestamp)
 
     dbsession.commit()
 
 
 
+
 class DBSessionInjector(object):
-    def __init__(self, sub_app, dburl):
+    def __init__(self, sub_app, dburl, echo=False):
         from sqlalchemy import create_engine
-        self.engine = create_engine(dburl, echo=True)
+        self.engine = create_engine(dburl)
         from synoptic.datamodel import DataModel
         self.datamodel = DataModel()
         self.datamodel.metadata.create_all(self.engine) 
@@ -114,6 +98,34 @@ class DBSessionInjector(object):
 
 
 
+class ErrorMiddleware(object):
+    def __init__(self, sub_app):
+        self.sub_app = sub_app
+
+    def __call__(self, environ, start_response):
+        import sys
+
+        from paste.httpexceptions import HTTPException
+
+        try:
+            return list(self.sub_app(environ, start_response))
+        except HTTPException, e:
+            return e(environ, start_response)
+        except:
+            status = "500 Server Error"
+            response_headers = [("content-type","text/plain")]
+            exc_info = sys.exc_info()
+            ex_type, ex_val, ex_tb = exc_info
+
+            from traceback import print_exc
+            try:
+                print_exc()
+            except:
+                pass
+
+            start_response(status, response_headers, exc_info)
+            return ["%s: %s" % (ex_type.__name__,str(ex_val))]
+        
 from paste.wsgiwrappers import WSGIRequest, WSGIResponse
 
 
@@ -148,7 +160,7 @@ class Request(WSGIRequest):
 
 
         
-class ApplicationBase:
+class ApplicationBase(object):
     def __init__(self, table):
         import re
         self.table = [
@@ -172,24 +184,29 @@ class ApplicationBase:
 
 
 
-
 class Application(ApplicationBase):
     def __init__(self):
         ApplicationBase.__init__(self,
                 [
                     (r'^/$', self.index),
-                    (r'^/timestamp/get_range$', self.get_tsrange),
-                    (r'^/item/get_by_id$', self.get_item_by_id),
-                    (r'^/item/get_multi_by_tags$', self.get_items_by_tags),
-                    (r'^/item/print_multi_by_tags$', self.print_items_by_tags),
-                    (r'^/item/export_multi_by_tags$', self.export_items_by_tags),
-                    (r'^/item/store$', self.store_item),
-                    (r'^/item/reorder$', self.reorder_item),
-                    (r'^/tags/get$', self.get_tags),
+                    (r'^/timestamp/get_range$', self.http_get_tsrange),
+                    (r'^/item/get_by_id$', self.http_get_item_by_id),
+                    (r'^/items/get$', self.http_get_items),
+                    (r'^/items/print$', self.http_print_items),
+                    (r'^/items/export$', self.http_export_items),
+                    (r'^/item/store$', self.http_store_item),
+                    (r'^/item/reorder$', self.http_reorder_item),
+                    (r'^/tags/get$', self.http_get_tags),
+                    (r'^/tags/rename$', self.http_rename_tag),
+                    (r'^/app/quit$', self.http_quit),
                     (r'^/static/([-_/a-zA-Z0-9.]+)$', self.serve_static),
                     ])
 
         WSGIRequest.defaults["charset"] = "utf-8"
+        self.quit_func = None
+
+    def set_quit_func(self, quit_func):
+        self.quit_func = quit_func
 
     # tools -------------------------------------------------------------------
     def item_to_json(self, item):
@@ -199,70 +216,9 @@ class Application(ApplicationBase):
         result['title'] = None
         return result
 
-    def get_current_versions_query(self, model, max_timestamp=None):
-        """Find the current version of all items."""
-
-        from sqlalchemy.sql import select, and_, or_, not_, func
-
-        result = select(
-                [model.itemversions.c.id, 
-                    func.max(model.itemversions.c.timestamp)])
-                
-        if max_timestamp is not None:
-            result = result.where(model.itemversions.c.timestamp <= max_timestamp)
-
-        result = result.group_by(model.itemversions.c.item_id)
-        return result.alias("current_versions")
-
-    def get_itemversions_query(self, session, model, tags, max_timestamp=None):
-        """Given tags and max_timestamp, find the resulting ItemVersion ids,
-        in the right order."""
-
-        from sqlalchemy.sql import select, and_, or_, not_, func
-
-        # find appropriate versions of items
-        current_query = self.get_current_versions_query(model, max_timestamp)
-
-        # find view ordering
-        view_orderings = (session.query(ViewOrdering)
-                .filter_by(tagset=ViewOrdering.make_tagset(tags))
-                .order_by(ViewOrdering.timestamp.desc())
-                .limit(1)).all()
-
-        # out of that, find all that have matching tags and are not deleted
-        from_obj = model.itemversions.join(current_query,
-                        current_query.c.id==model.itemversions.c.id)
-
-        if view_orderings:
-            # if we have an ordering, we must also join the viewordering_entries table
-            # so we can order by weight
-
-            vo_entries = (
-                    select([model.viewordering_entries])
-                    .where(model.viewordering_entries.c.viewordering_id
-                        ==view_orderings[0].id)).alias("vo_entries")
-
-            from_obj = from_obj.outerjoin(vo_entries,
-                    model.itemversions.c.item_id==vo_entries.c.item_id)
-                        
-        tag_where = model.itemversions.c.contents != None
-        for tag in tags:
-            tag_where = and_(tag_where, ItemVersion.tags.any(id=tag.id))
-
-        result = (select(
-            [model.itemversions], 
-            from_obj=[from_obj])
-            .where(tag_where))
-
-        if view_orderings:
-            # add the ordering clause
-            result = result.order_by(vo_entries.c.weight)
-
-        return result.group_by(model.itemversions.c.item_id)
-
-    def get_itemversions_for_tag(self, request):
-        tags = find_tags(request.dbsession, request.GET.get("query", ""), 
-                create_them=False)
+    def get_itemversions_for_request(self, request):
+        from synoptic.query import parse_query
+        parsed_query = parse_query(request.GET.get("query", ""))
 
         if "max_timestamp" in request.GET:
             max_timestamp = float(request.GET["max_timestamp"])
@@ -275,23 +231,12 @@ class Application(ApplicationBase):
         # grab the ORMed instances
         return (session
                 .query(ItemVersion)
-                .from_statement(self.get_itemversions_query(
-                    session, model, tags, max_timestamp)))
+                .from_statement(query_itemversions(
+                    session, model, parsed_query, max_timestamp)))
 
-    def get_json_items_for_request(self, request):
-        tags = find_tags(request.dbsession, request.GET["query"], 
-                create_them=False)
-
-        if "max_timestamp" in request.GET:
-            max_timestamp = float(request.GET["max_timestamp"])
-        else:
-            max_timestamp = None
-
-        session = request.dbsession
-        model = request.datamodel
-
-        itemversions_query = self.get_itemversions_query(
-                session, model, tags, max_timestamp).alias("currentitemversions")
+    def get_json_items(self, session, model, parsed_query, max_timestamp):
+        itemversions_query = query_itemversions(
+                session, model, parsed_query, max_timestamp).alias("currentitemversions")
 
         # prepare eager loading of tags
         from sqlalchemy.sql import select
@@ -330,7 +275,7 @@ class Application(ApplicationBase):
         ctx = Context()
         return request.respond(mainpage(ctx))
 
-    def get_tsrange(self, request):
+    def http_get_tsrange(self, request):
         from time import time
 
         now = time()
@@ -346,59 +291,102 @@ class Application(ApplicationBase):
                     }),
                 mimetype="text/plain")
 
-    def get_tags(self, request):
+    def get_tags_with_usecounts(self, session, model, parsed_query=None, 
+            max_timestamp=None):
         from sqlalchemy.sql import select, and_, or_, not_, func
 
-        model = request.datamodel
-
-        if "max_timestamp" in request.GET:
-            max_timestamp = float(request.GET["max_timestamp"])
+        if parsed_query is not None:
+            itemversions_q = query_itemversions(session, model,
+                    parsed_query, max_timestamp)
         else:
-            max_timestamp = None
-
-        current_query = self.get_current_versions_query(model, max_timestamp)
+            itemversions_q = select([model.itemversions],
+                    from_obj=[get_current_itemversions_join(
+                        model, max_timestamp)
+                        ])
+                    
+        itemversions_q = itemversions_q.alias("currentversions")
 
         # twuc_q stands for tags_with_usecount_query
         twuc_q = (
                 select([
                     model.tags.c.name, 
-                    func.count(current_query.c.id).label("use_count"),
+                    func.count(itemversions_q.c.id).label("use_count"),
                         ],
                     from_obj=[model.itemversions_tags
-                        .join(current_query, 
-                            current_query.c.id==model.itemversions_tags.c.itemversion_id)
+                        .join(itemversions_q, 
+                            itemversions_q.c.id==model.itemversions_tags.c.itemversion_id)
                         .join(model.tags)
                         ])
                 )
-        if "q" in request.GET:
-            twuc_q = twuc_q.where(model.tags.c.name.like('%s%%' % request.GET["q"]))
 
         twuc_q = (twuc_q
                 .group_by(model.tags.c.id)
-                .having(func.count(current_query.c.id)>0)
+                .having(func.count(itemversions_q.c.id)>0)
                 .order_by(model.tags.c.name)
                 )
-        if "limit" in request.GET:
-            twuc_q = twuc_q.limit(int(request.GET["limit"]))
 
-        result = request.dbsession.execute(twuc_q)
+        return session.execute(twuc_q)
 
-        if "withusecount" in request.GET:
-            from simplejson import dumps
-
-            data = [list(row) for row in result]
-
-            return request.respond(
-                    dumps({ 
-                    "data": data,
-                    "max_usecount": max([0] + [row[1] for row in data]),
-                    }),
-                    mimetype="text/plain")
+    def http_get_tags(self, request):
+        if "max_timestamp" in request.GET:
+            max_timestamp = float(request.GET["max_timestamp"])
         else:
-            return request.respond(u"\n".join(row[0] for row in result), 
-                    mimetype="text/plain")
+            max_timestamp = None
 
-    def get_item_by_id(self, request):
+        query = request.GET.get("query", "")
+
+        if query != "":
+            from synoptic.query import parse_query
+            parsed_query = parse_query(query)
+
+            from synoptic.query import TagListVisitor
+            query_tags = parsed_query.visit(TagListVisitor())
+        else:
+            parsed_query = None
+            query_tags = []
+
+        result = self.get_tags_with_usecounts(
+                request.dbsession, request.datamodel, parsed_query,
+                max_timestamp)
+
+        from simplejson import dumps
+
+        tags = [list(row) for row in result]
+
+        return request.respond(
+                dumps({ 
+                "tags": tags,
+                "query_tags": query_tags,
+                }),
+                mimetype="text/plain")
+
+    def http_rename_tag(self, request):
+        from simplejson import loads, dumps
+        data = loads(request.POST["json"])
+
+        old_name = data["old_name"]
+        new_name = data["new_name"]
+
+        tag = request.dbsession.query(Tag).filter_by(name=old_name).one()
+        new_tag_query = request.dbsession.query(Tag).filter_by(name=new_name)
+        
+        if new_tag_query.count():
+            raise ValueError, "tag already exsits"
+
+        tag.name = new_name
+
+        import re
+
+        old_re = re.compile(r"\b%s\b" % re.escape(old_name))
+
+        for v_ord in request.dbsession.query(ViewOrdering).filter(
+                ViewOrdering.tagset.contains(old_name)):
+            v_ord.tagset = old_re.sub(new_name, v_ord.tagset)
+            print v_ord.tagset
+
+        return request.respond("", mimetype="text/plain")
+
+    def http_get_item_by_id(self, request):
         query = (
                 request.dbsession.query(ItemVersion)
                 .filter_by(item_id=int(request.GET["id"]))
@@ -415,38 +403,71 @@ class Application(ApplicationBase):
         return request.respond(dumps(self.item_to_json(query.first())),
                 mimetype="text/plain")
 
-    def get_items_by_tags(self, request):
-        json_items = self.get_json_items_for_request(request)
+    def http_get_items(self, request):
+        qry = request.GET.get("query", "")
 
-        tag_counter = {}
-        for it in json_items:
-            for tag in it["tags"]:
-                tag_counter[tag] = tag_counter.get(tag, 0) + 1
+        from synoptic.query import parse_query
+        parsed_query = parse_query(qry)
 
-        tags = [list(it) for it in tag_counter.items()]
-        tags.sort()
+        if "max_timestamp" in request.GET:
+            max_timestamp = float(request.GET["max_timestamp"])
+        else:
+            max_timestamp = None
+
+        json_items = self.get_json_items(
+                request.dbsession, request.datamodel, 
+                parsed_query, max_timestamp)
+
+        if qry != "":
+            tag_counter = {}
+            for it in json_items:
+                for tag in it["tags"]:
+                    tag_counter[tag] = tag_counter.get(tag, 0) + 1
+
+            tags = [list(it) for it in tag_counter.iteritems()]
+            tags.sort()
+
+            from synoptic.query import TagListVisitor
+            query_tags = parsed_query.visit(TagListVisitor())
+        else:
+            # the empty query parses as "home", but we need to show
+            # all tags, not just the ones relative to that,
+
+            if "max_timestamp" in request.GET:
+                max_timestamp = float(request.GET["max_timestamp"])
+            else:
+                max_timestamp = None
+
+            tags = [list(row) 
+                    for row in self.get_tags_with_usecounts(
+                        request.dbsession, request.datamodel,
+                        max_timestamp=max_timestamp)]
+
+            query_tags = []
 
         # and ship them out by JSON
         from simplejson import dumps
         return request.respond(dumps({
             "items": json_items,
             "tags": tags,
-            "max_usecount": max([0]+tag_counter.values()),
+            "query_tags": query_tags,
             }),
                 mimetype="text/plain")
 
-    def print_items_by_tags(self, request):
-        versions = self.get_itemversions_for_tag(request)
+    def http_print_items(self, request):
+        versions = self.get_itemversions_for_request(request)
 
         from html import printpage
         return request.respond(
                 printpage({
                     "title": "Synoptic Printout",
-                    "body": "<hr/>".join(v.contents_html() for v in versions),
+                    "body": "<hr/>".join(
+                        '<div class="itemcontents">%s</div>' % v.contents_html() 
+                        for v in versions),
                     }))
 
-    def export_items_by_tags(self, request):
-        versions = self.get_itemversions_for_tag(request)
+    def http_export_items(self, request):
+        versions = self.get_itemversions_for_request(request)
 
         sep = (75*"-") + "\n"
         return request.respond(
@@ -457,62 +478,69 @@ class Application(ApplicationBase):
                     for v in versions),
                 mimetype="text/plain;charset=utf-8")
 
-    def store_item(self, request):
+    def http_store_item(self, request):
         from simplejson import loads, dumps
         data = loads(request.POST["json"])
 
+        # if view ordering is present for current query,
+        # make sure this entry shows up last
+        if data["contents"] is not None:
+            # if we're not deleting
+            from synoptic.datamodel import ViewOrderingHandler
+            from synoptic.query import parse_query
+            voh = ViewOrderingHandler(
+                    request.dbsession, request.datamodel,
+                    parse_query(data["current_query"]))
+            if voh.has_ordering():
+                voh.load()
+                if data["id"] in voh:
+                    # we already have a spot in the ordering, don't bother
+                    voh = None
+            else:
+                voh = None
+        else:
+            voh = None
+
         itemversion = store_itemversion(request.dbsession, 
                 data["contents"], data["tags"], data["id"])
 
-        request.dbsession.commit() # fills in the item id
+        request.dbsession.commit() # fills in the item_id
 
+        if voh is not None:
+            voh.insert(len(voh), itemversion.item_id)
+            voh.save()
+
+        # send response
         from simplejson import dumps
         return request.respond(
                 dumps(self.item_to_json(itemversion)),
                 mimetype="text/plain")
 
-    def reorder_item(self, request):
+    def http_quit(self, request):
+        if self.quit_func is not None:
+            self.quit_func()
+            return request.respond(
+                    "Thanks for using synoptic.", 
+                    mimetype="text/plain")
+        else:
+            raise HTTPForbidden()
+
+    def http_reorder_item(self, request):
         from simplejson import loads
         data = loads(request.POST["json"])
 
-        tags = find_tags(request.dbsession, data["current_search"], create_them=False)
+        from synoptic.query import parse_query
+        from synoptic.datamodel import ViewOrderingHandler
 
-        model = request.datamodel
-        session = request.dbsession
-
-        item_ids = [row[model.itemversions.c.item_id]
-                for row in session.execute(
-                self.get_itemversions_query(session, model, tags))]
-
-        def item_idx(sought_item_id):
-            if sought_item_id == None:
-                return len(item_ids)
-            for idx, item_id in enumerate(item_ids):
-                if item_id == sought_item_id:
-                    return idx
-            raise ValueError, "invalid item id supplied"
-
-        dragged_item_idx = item_idx(data["dragged_item"])
-        before_item_idx = item_idx(data["before_item"])
-
-        assert dragged_item_idx != before_item_idx
-
-        if before_item_idx < dragged_item_idx:
-            dragged_item = item_ids.pop(dragged_item_idx)
-            item_ids.insert(before_item_idx, dragged_item)
-        else:
-            item_ids.insert(before_item_idx, item_ids[dragged_item_idx])
-            item_ids.pop(dragged_item_idx)
-
-        from time import time
-
-        viewordering = ViewOrdering(ViewOrdering.make_tagset(tags), time())
-
-        for idx, item_id in enumerate(item_ids):
-            viewordering.entries.append(ViewOrderingEntry(viewordering, 
-                session.query(Item).get(item_id), idx))
-        session.save(viewordering)
-        session.commit()
+        voh = ViewOrderingHandler(
+                request.dbsession, request.datamodel,
+                parse_query(data["current_search"]))
+        voh.load()
+        voh.reorder(
+                voh.index_from_id(data["dragged_item"]),
+                voh.index_from_id(data["before_item"]),
+                )
+        voh.save()
 
         return request.respond("", mimetype="text/plain")
 
