@@ -14,12 +14,14 @@ def printpage(context):
     return """
     <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"
          "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
+    <html>
     <head>
       <title>%(title)s</title>
+      <link rel="stylesheet" type="text/css" href="/static/content.css">
     </head>
-      <body>
+    <body>
         %(body)s
-      </body>
+    </body>
     </html>
     """ % context
 
@@ -31,13 +33,14 @@ def page(context):
     return """
     <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"
          "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
+    <html>
     <head>
       <title>%(title)s</title>
       <script type="text/javascript" src="/static/jquery.js"></script>
       <script type="text/javascript" src="/static/jquery.timers.js"></script>
       <script type="text/javascript" src="/static/jquery.bgiframe.js"></script>
       <script type="text/javascript" src="/static/jquery.dimensions.js"></script>
-      <script type="text/javascript" src="/static/jquery.autocomplete.js"></script>
+      <script type="text/javascript" src="/static/jquery.contextmenu.r2.js"></script>
       <script type="text/javascript" src="/static/jquery-ui/ui.base.js"></script>
       <script type="text/javascript" src="/static/jquery-ui/ui.mouse.js"></script>
       <script type="text/javascript" src="/static/jquery-ui/ui.slider.js"></script>
@@ -49,13 +52,15 @@ def page(context):
       <script type="text/javascript" src="/static/jquery-ui/datepicker/core/ui.datepicker.js"></script>
       <script type="text/javascript" src="/static/inheritance.js"></script>
       <script type="text/javascript" src="/static/json2.js"></script>
+      <script type="text/javascript" src="/static/rsh.js"></script>
       <script type="text/javascript" src="/static/main.js"></script>
+      <link rel="stylesheet" type="text/css" href="/static/content.css">
       <link rel="stylesheet" type="text/css" href="/static/style.css">
       <link rel="stylesheet" href="/static/jquery-ui/themes/flora/flora.all.css" type="text/css" media="screen">
     </head>
-      <body>
+    <body>
         %(body)s
-      </body>
+    </body>
     </html>
     """ % context
 
@@ -84,22 +89,19 @@ def navpane(context):
         <div id="navtabs" class="flora">
           <ul>
             <li><a href="#fragment-1"><span>Tags</span></a></li>
-            <li><a href="#fragment-2"><span>Sub-Tags</span></a></li>
-            <li><a href="#fragment-3"><span>Search</span></a></li>
+            <li><a href="#fragment-2"><span>All Tags</span></a></li>
           </ul>
           <div id="fragment-1">
+            <div id="subtagcloud_search_tags"> </div>
+            <div id="subtagcloud"> </div>
+            <input type="checkbox" id="chk_subtagcloud_show_hidden"/>
+            <label for="chk_subtagcloud_show_hidden">Show hidden (.<i>tag</i>)</label>
+          </div>
+          <div id="fragment-2">
             <div id="tagcloud"> </div>
             <input type="checkbox" id="chk_tagcloud_show_hidden"/>
             <label for="chk_tagcloud_show_hidden">Show hidden (.<i>tag</i>)</label>
           </div>
-          <div id="fragment-2">
-            <div id="subtagcloud_search_tags"> </div>
-            <div id="subtagcloud"> </div>
-          </div>
-          <div id="fragment-3">
-            <div id="viewlist"> </div>
-          </div>
-          </div>
         </div>
       </div>
     </div>
@@ -110,19 +112,35 @@ def mainpane(context):
     <div id="mainpane">
       <div id="innermainpane">
         <div style="white-space:nowrap;">
-          <label for="search" accesskey="s">Search: </label>
           <input type="text" size="50" id="search"/>
-          <img src="/static/edit-clear.png" alt="Clear search bar" id="btn_search_clear" class="imagebutton"/>
-          <img src="/static/print.png" alt="Print" id="btn_print" class="imagebutton"/>
+          <img src="/static/edit-clear.png" alt="Clear search bar" title="Clear search bar" id="btn_search_clear" class="imagebutton"/>
+          <img src="/static/print.png" alt="Print" title="Print" id="btn_print" class="imagebutton"/>
+          <img src="/static/expand-all.png" alt="Expand all" title="Expand all" id="btn_expand" class="imagebutton"/>
+          <img src="/static/collapse-all.png" alt="Collapse all" title="Collapse all" id="btn_collapse" class="imagebutton"/>
+          <img src="/static/export.png" alt="Export" title="Export" id="btn_export" class="imagebutton"/>
+          <img src="/static/quit.png" alt="Quit" title="Quit" id="btn_quit" class="imagebutton"/>
         </div>
-        <div id="items>
+        <div id="items">
         </div>
       </div>
     </div>
     """
 
+def context_menus():
+    return """
+    <div id="tag_context_menu">
+      <ul>
+        <li id="rename">Rename</li>
+        <li id="restrict">Restrict search</li>
+        <li id="search">Search for only this tag</li>
+      </ul>
+    </div>
+    """
+
 def mainpage(context):
     return page(context.add(
         title="Synoptic",
-        body='<div id="main_container">%s</div>' % (navpane(context)+mainpane(context))
+        body='<div id="main_container">%s</div>%s' % (
+            navpane(context)+mainpane(context),
+            context_menus())
         ))
