@@ -59,6 +59,34 @@ function str(val)
 
 
 
+// misc tools -----------------------------------------------------------------
+function re_escape(text) {
+  if (!arguments.callee.sRE) {
+    var specials = [
+      '/', '.', '*', '+', '?', '|',
+      '(', ')', '[', ']', '{', '}', '\\'
+    ];
+    arguments.callee.sRE = new RegExp(
+      '(\\' + specials.join('|\\') + ')', 'g'
+    );
+  }
+  return text.replace(arguments.callee.sRE, '\\$1');
+}
+
+
+
+
+function find_in_array(ary, item)
+{
+  for (var i = 0; i < ary.length; ++i)
+    if (ary[i] == item)
+      return i;
+  return -1;
+}
+
+
+
+
 // ItemManager  --------------------------------------------------------------------
 function ItemManager(mgr, arg)
 {
@@ -92,8 +120,9 @@ ItemManager.method("set_from_obj", function(arg)
 
 ItemManager.method("call_with_item_div", function(inserter, when_created)
 {
-  inserter('<div id="item_[id]"></div>'.replace('[id]', this.id));
+  inserter('<div id="item_[id]" class="item"></div>'.replace('[id]', this.id));
   this.div = $("#item_"+this.id);
+
   if (when_created != undefined)
     when_created(this.div);
   this.fill_item_div();
@@ -114,7 +143,7 @@ ItemManager.method("fill_item_div", function()
       (
       '<div id="item_dropzone_[id]" class="dropzone">Drop here</div>'+
       '<div class="editcontrols">'+
-      '<div id="item_cursor_[id]" class="cursorfield"></div> '+
+      '<span id="item_cursor_[id]" class="cursorfield">&nbsp;</span> '+
       '<input type="button" id="new_[id]" value="New" class="editbutton"/>'+
       '</div>'
       ).allreplace('[id]', self.id)
@@ -130,11 +159,14 @@ ItemManager.method("fill_item_div", function()
         (
         '<div id="item_dropzone_[id]" class="dropzone">Drop here</div>'+
         '<div class="editcontrols">'+
-        '<div id="item_cursor_[id]" class="cursorfield"></div> '+
-        '<a id="item_collapser_[id]" class="collapser">-</a> '+
-        '<div id="item_draghandle_[id]" class="draghandle">&uarr;&darr;</div> '+
+        '<span id="item_cursor_[id]" class="cursorfield">&nbsp;</span> '+
+        '<span id="item_collapser_[id]" class="collapser">&nbsp;</span> '+
         '<input type="button" id="edit_[id]" value="Edit" class="editbutton"/> '+
         '<input type="button" id="delete_[id]" value="Delete" class="deletebutton"/> '+
+        '<div id="item_draghandle_[id]" class="draghandle">'+
+          '<img src="/static/dragger.png" alt="drag handle" '+
+          'title="Drag this handle to change display order"/>'+
+        '</div> '+
         'Tags: [tags]'+
         '</div>'+
         '<div id="item_contents_[id]" class="itemcontents">[contents]</div>'
@@ -162,8 +194,8 @@ ItemManager.method("fill_item_div", function()
       self.div.html(
         (
         '<div class="editcontrols">'+
-        '<div id="item_cursor_[id]" class="cursorfield"></div> '+
-        '<a id="item_collapser_[id]" class="collapser">-</a> '+
+        '<span id="item_cursor_[id]" class="cursorfield">&nbsp;</span> '+
+        '<span id="item_collapser_[id]" class="collapser">&nbsp;</span> '+
         '<input type="button" id="btn_revert_[id]" value="Revert"/> '+
         '<input type="button" id="btn_copy_to_present_[id]" value="Copy to Present"/> '+
         'Tags: [tags]'+
@@ -182,12 +214,13 @@ ItemManager.method("fill_item_div", function()
             data: {json: JSON.stringify({
               id: self.id,
               tags: self.tags,
-              contents: self.contents
+              contents: self.contents,
+              current_query: $("#search").val()
             })},
             error: function(req, stat, err) { report_error("Revert failed."); },
             success: function(data, msg) { 
               set_message("Revert successful."); 
-              update_tag_cloud();
+              update_tag_clouds();
             }
           });
         });
@@ -200,36 +233,24 @@ ItemManager.method("fill_item_div", function()
             data: {json: JSON.stringify({
               id: null,
               tags: self.tags,
-              contents: self.contents
+              contents: self.contents,
+              current_query: $("#search").val()
             })},
             error: function(req, stat, err) { report_error("Copy failed."); },
             success: function(data, msg) { 
               set_message("Copy successful."); 
-              update_tag_cloud();
+              update_tag_clouds();
             }
           });
         });
     }
 
     var query = "#item_[id] div.editcontrols a.taglink".replace("[id]", self.id);
-    make_tag_links($(query));
+    add_tag_behavior($(query));
 
     $("#item_collapser_"+self.id).click(
-        function()
-        {
-          if (self.hidden)
-          {
-            $("#item_collapser_"+self.id).html("-");
-            $("#item_contents_"+self.id).show("slow");
-            self.hidden = false;
-          }
-          else
-          {
-            $("#item_collapser_"+self.id).html("+");
-            $("#item_contents_"+self.id).hide("slow");
-            self.hidden = true;
-          }
-        });
+        function() { item_toggle_collapsed(self.div); }
+        );
   }
 
   if (have_dropzone)
@@ -260,13 +281,14 @@ ItemManager.method("fill_item_div", function()
           type: 'POST',
           dataType: 'text',
           url: '/item/reorder',
-          data: {json: JSON.stringify({
-            dragged_item: dragged_item.id,
-            before_item: self.id,
-            current_search: parse_tags($("#search").val()),
-          })},
+          data: {
+            json: JSON.stringify({
+              dragged_item: dragged_item.id,
+              before_item: self.id,
+              current_search: $("#search").val()
+            })},
           error: function(req, stat, err) 
-          { report_error("Reordering failed on server."); },
+          { report_error("Reordering failed on server."); }
         });
       }
       });
@@ -274,6 +296,13 @@ ItemManager.method("fill_item_div", function()
 });
 
 
+function item_toggle_collapsed(div)
+{
+  if ($(div).hasClass("collapsed"))
+    $(div).removeClass("collapsed");
+  else
+    $(div).addClass("collapsed");
+}
 
 
 ItemManager.method("begin_edit", function() 
@@ -297,20 +326,24 @@ ItemManager.method("begin_edit", function()
     '<textarea id="editor_[id]" cols="80"></textarea>'+
     '</td></tr>'+
     '</tr></table>'+
-    '</div>'+
-    '<div  id="edit_errors_[id]"  class="errors"></div>'
+    '</div>'
     ).allreplace("[id]",  self.id)
     );
 
   // default to current search tags
   if (self.id == null)
-    $("#edit_tags_"+self.id).val($("#search").val());
+  {
+    var search_val = $("#search").val();
+    if (search_val == "")
+      $("#edit_tags_"+self.id).val("home");
+    else
+      $("#edit_tags_"+self.id).val(search_val);
+  }
   else
     $("#edit_tags_"+self.id).val(self.tags);
 
   $("#editor_"+self.id).val(self.contents);
 
-  // focus after a delay
   self.manager.install_focus_handlers($("#editor_"+self.id));
   $("#editor_"+self.id).get(0).focus();
   $("#editor_"+self.id).height(text_height);
@@ -323,28 +356,42 @@ ItemManager.method("begin_edit", function()
 
   $("#edit_ok_"+self.id).click(function(){
     $("edit_div_"+self.id).html(busy('Saving...'));
+    var tags = parse_tags($("#edit_tags_"+self.id).val());
+
+    for (var i = 0; i<tags.length; ++i)
+      if (!is_valid_tag(tags[i]))
+      {
+        alert("Invalid tag: '[tag]'".allreplace("[tag]", tags[i]));
+        return;
+      }
+
+    self.contents = $("#editor_"+self.id).val();
+
     $.ajax({
       type: 'POST',
       dataType: 'json',
       url: '/item/store',
       data: {json: JSON.stringify({
         id: self.id,
-        tags: parse_tags($("#edit_tags_"+self.id).val()),
-        contents: $("#editor_"+self.id).val()
+        tags: tags,
+        contents: self.contents,
+        current_query: $("#search").val()
       })},
-      error: function(req, stat, err) {
+
+      error: function(req, stat, err) 
+      {
+        set_message("Saving failed.");
         self.begin_edit();
-        $("#edit_errors_"+self.id).html(req.responseText);
       },
+
       success: function(data, msg) 
       {
         var prev_id = self.id;
-        $("#edit_errors_"+self.id).html('');
         self.set_from_obj(data);
         self.id = data.id;
         self.call_with_item_div(function(html){ self.div.replaceWith(html); });
         self.manager.set_cursor_to(self.div);
-        update_tag_cloud();
+        update_tag_clouds();
         if (prev_id == null)
         {
           self.manager.empty_was_filled();
@@ -380,7 +427,7 @@ ItemManager.method("do_delete", function()
     })},
     error: function(req, stat, err) { report_error("Delete failed."); },
     success: function(data, msg) { 
-      update_tag_cloud();
+      update_tag_clouds();
     }
   });
 });
@@ -419,25 +466,49 @@ function ItemCollectionManager()
   self.setup_history_handling();
   self.setup_search();
   self.setup_keyboard_nav();
+  self.setup_toolbar();
 }
 
 
 
 
+ItemCollectionManager.method("set_history_slider", function(time)
+{
+  var new_percentage;
+  if (time == null || this.tsrange == undefined)
+    new_percentage = 100.;
+  else
+    new_percentage = (time-this.tsrange.min)
+      / (this.tsrange.max-this.tsrange.min)* 100.;
+
+  ++this.slider_update_inhibitions;
+  $("#history_slider").slider("moveTo", new_percentage);
+  --this.slider_update_inhibitions;
+});
+
+
+
+
 ItemCollectionManager.method("setup_history_handling", function()
 {
   var self = this;
 
   self.tsrange = null;
   $.getJSON("/timestamp/get_range", function (json)
-    { self.tsrange = json; });
+    { 
+      self.tsrange = json; 
+      self.set_history_slider(self.view_time);
+    });
+
   self.view_time = null;
 
   // setup history slider
+  self.slider_update_inhibitions = 0;
+
   $("#history_slider").slider({
       slide: function(e, ui)
         {
-          if (self.tsrange == null)
+          if (self.tsrange == null || self.slider_update_inhibitions)
             return;
 
           if (ui.value == 100)
@@ -449,17 +520,19 @@ ItemCollectionManager.method("setup_history_handling", function()
         },
       change: function(e, ui)
         {
-          if (self.tsrange == null)
+          if (self.tsrange == null || self.slider_update_inhibitions)
             return;
 
           if (ui.value == 100)
             self.set_time(null, "slider");
           else
+          {
             var new_time = self.tsrange.min 
               + ui.value/100.*(self.tsrange.max-self.tsrange.min);
             self.set_time(new_time, "slider");
+          }
         },
-      startValue: 100,
+      startValue: 100
       });
 
   // setup history datepicker
@@ -486,6 +559,42 @@ ItemCollectionManager.method("setup_history_handling", function()
 
 
 
+ItemCollectionManager.method("add_history_item", function()
+{
+  dhtmlHistory.add(
+    escape(JSON.stringify({
+      query:$("#search").val(),
+      timestamp:this.view_time
+    })));
+});
+
+
+
+
+ItemCollectionManager.method("handle_history_event", function(new_loc, dummy)
+{
+  if (new_loc != "")
+  {
+    new_loc = unescape(new_loc);
+    if (new_loc[0] == "{")
+    {
+      var loc_descriptor = JSON.parse(new_loc);
+      $("#search").val(loc_descriptor.query);
+      this.set_time(loc_descriptor.timestamp, "history");
+    }
+    else
+      set_search(new_loc, true);
+  }
+  else
+  {
+    $("#search").val("");
+    this.set_time(null, "history");
+  }
+});
+
+
+
+
 ItemCollectionManager.method("setup_search", function()
 {
   var self = this;
@@ -496,12 +605,41 @@ ItemCollectionManager.method("setup_search", function()
     });
   $("#search").focus(function()
     {
-      self.div.everyTime("250ms", "search_changewatch",
-        function() { self.update(); });
+      var last_search = $("#search").val();
+      var unchanged_count = 0;
+      var is_updated = true;
+
+      self.div.everyTime("100ms", "search_changewatch",
+        function() 
+        { 
+          var search = $("#search").val();
+          if (search != last_search)
+          {
+            is_updated = false;
+            last_search = search;
+            unchanged_count = 0;
+            return
+          }
+
+          if (!is_updated)
+          {
+            ++unchanged_count;
+
+            if (unchanged_count > 3)
+            {
+              self.update(); 
+              is_updated = true;
+            }
+          }
+        });
+      ++self.search_focused;
     });
   $("#search").blur(function()
     {
       self.div.stopTime("search_changewatch");
+      self.add_history_item();
+      self.update(); 
+      --self.search_focused;
     });
   /*
   $("#search").autocomplete("/tags/get",
@@ -509,8 +647,7 @@ ItemCollectionManager.method("setup_search", function()
       */
   $("#btn_search_clear").click(function()
     {
-      $("#search").val('');
-      $("#search").change();
+      set_search("");
       $("#search").get(0).focus();
     });
 
@@ -524,19 +661,29 @@ ItemCollectionManager.method("setup_keyboard_nav", function()
 {
   var self = this;
   self.cursor_at = null;
-  self.kb_handler_inhibitions = 0;
+  self.kb_shortcut_inhibitions = 0;
+  self.search_focused = 0;
   
   $(document).keypress(function(ev)
     {
       var key = String.fromCharCode(ev.charCode);
       var handled = false;
 
-      if (self.kb_handler_inhibitions != 0)
+      if (self.search_focused && ev.keyCode == 13)
+      {
+        $("#search").get(0).blur();
+        return;
+      }
+
+      if (self.kb_shortcut_inhibitions != 0)
         return true;
 
-      if (key == "s")
+      if (key == "s" || key =="/")
         $("#search").get(0).focus();
 
+      if (key == "c")
+        self.cursor_at.find(".editcontrols .collapser").click();
+
       if (self.cursor_at != null)
       {
         if (key == "j")
@@ -575,6 +722,41 @@ ItemCollectionManager.method("setup_keyboard_nav", function()
 
 
 
+ItemCollectionManager.method("setup_toolbar", function()
+{
+  $("#btn_print").click(function()
+    {
+      var printurl = "/items/print?query="+escape($("#search").val());
+      if (self.view_time != null)
+        printurl += "&max_timestamp="+escape(self.view_time.toString());
+
+      window.open(printurl);
+    });
+  $("#btn_export").click(function()
+    {
+      var exporturl = "/items/export?query="+escape($("#search").val());
+      if (self.view_time != null)
+        exporturl += "&max_timestamp="+escape(self.view_time.toString());
+
+      window.open(exporturl);
+    });
+  $("#btn_expand").click(function()
+    { 
+      $(".item").removeClass("collapsed");
+    });
+  $("#btn_collapse").click(function()
+    { 
+      $(".item").addClass("collapsed");
+    });
+  $("#btn_quit").click(function()
+    { 
+      location.href = "/app/quit";
+    });
+});
+
+
+
+
 ItemCollectionManager.method("redraw_cursor", function()
 {
   if (this.cursor_at != null)
@@ -584,17 +766,16 @@ ItemCollectionManager.method("redraw_cursor", function()
 
 
 
-ItemCollectionManager.method("set_cursor_to", function(div, is_up)
+ItemCollectionManager.method("set_cursor_to", function(div, is_up, noscroll)
 {
   if (this.cursor_at != null)
   {
-    this.cursor_at.find(".editcontrols .cursorfield").html("");
     this.cursor_at.find(".editcontrols").removeClass("hascursor");
   }
 
-  div.find(".editcontrols .cursorfield").html(">");
   div.find(".editcontrols").addClass("hascursor");
-  div.get(0).scrollIntoView(is_up);
+  if (!noscroll)
+    div.get(0).scrollIntoView(is_up);
   this.cursor_at = div;
 });
 
@@ -606,11 +787,11 @@ ItemCollectionManager.method("install_focus_handlers", function(input_el)
   var self = this;
   input_el.focus(function(ev) 
     { 
-      ++self.kb_handler_inhibitions; 
+      ++self.kb_shortcut_inhibitions; 
     });
   input_el.blur(function(ev) 
     { 
-      --self.kb_handler_inhibitions; 
+      --self.kb_shortcut_inhibitions; 
     });
 });
 
@@ -627,7 +808,7 @@ ItemCollectionManager.method("note_list_emptied", function(item)
 
 ItemCollectionManager.method("note_new_first_item", function(item)
 {
-  this.set_cursor_to(item.div);
+  this.set_cursor_to(item.div, /* is_up */ true, /*noscroll*/ true);
 });
 
 
@@ -635,12 +816,6 @@ ItemCollectionManager.method("note_new_first_item", function(item)
 
 ItemCollectionManager.method("show_time", function(new_time, origin)
 {
-  if (this.tsrange == undefined)
-  {
-    report_error("Cannot set date until timestamp range has been received.");
-    return;
-  }
-
   var dt = null;
   if (new_time != null)
   {
@@ -664,23 +839,15 @@ ItemCollectionManager.method("show_time", function(new_time, origin)
 
 ItemCollectionManager.method("set_time", function(new_time, origin)
 {
-  if (this.tsrange == undefined)
-  {
-    report_error("Cannot set date until timestamp range has been received.");
-    return;
-  }
-
   this.view_time = new_time;
   this.update();
   this.show_time(new_time, origin);
 
+  if (origin != "history")
+    this.add_history_item();
+
   if (origin != "slider")
-  {
-    var new_percentage = (new_time-this.tsrange.min)
-      / (this.tsrange.max-this.tsrange.min)* 100.
-    this.set_time(new_time, "slider");
-    $("#history_slider").slider("moveTo", new_percentage.toInt());
-  }
+    this.set_history_slider(new_time);
 });
 
 
@@ -721,18 +888,24 @@ ItemCollectionManager.method("empty_was_filled", function()
 
 
 
-ItemCollectionManager.method("update", function()
+ItemCollectionManager.method("update", function(force)
 {
-  this.fill($("#search").val(), this.view_time);
+  this.fill($("#search").val(), this.view_time, force);
 })
 
 
 
 
-ItemCollectionManager.method("fill", function(query, timestamp)
+ItemCollectionManager.method("fill", function(query, timestamp, force)
 {
+  if ($(".item .edit_div").length != 0)
+  {
+    if (confirm("You seem to be editing one or more entries. Continue loading?") == false)
+      return;
+  }
+
   var self = this;
-  if (query == self.last_query && timestamp == self.last_timestamp)
+  if (!force && query == self.last_query && timestamp == self.last_timestamp)
     return;
 
   self.div.html(busy('Loading...'));
@@ -751,7 +924,7 @@ ItemCollectionManager.method("fill", function(query, timestamp)
 
   $.ajax({
     data: data,
-    url: '/item/get_multi_by_tags',
+    url: '/items/get',
     dataType:"json",
     success: function(json, status)
     {
@@ -767,18 +940,22 @@ ItemCollectionManager.method("fill", function(query, timestamp)
       }
       self.realize_items(items);
 
-      // generate sub-tag cloud
-      var search_tags = parse_tags($("#search").val());
-      $("#subtagcloud").html(
-        make_tag_cloud(json.tags, json.max_usecount, true, search_tags));
-      make_tag_links($("#subtagcloud a"));
-
-      $("#subtagcloud_search_tags").html(format_tag_links(search_tags, " &middot; "));
-      make_tag_links($("#subtagcloud_search_tags a"));
+      fill_subtag_cloud(json);
     },
     error: function(req, stat, err)
     {
-      self.div.html('<div class="error">An error occurred.</div>');
+      var msg = "Sorry, we encountered an error.";
+      var longmsg = ''
+      if (err != undefined)
+        msg = msg + ' (' + err + ')';
+      if (req.responseText != undefined)
+        longmsg = req.responseText;
+
+      self.div.html(('<div class="error"><div class="message">[msg]</div></div>'
+          +'<div class="errordetail">[longmsg]</div>')
+          .allreplace('[msg]', msg)
+          .allreplace('[longmsg]', longmsg)
+          );
     }
     });
 })
@@ -787,6 +964,14 @@ ItemCollectionManager.method("fill", function(query, timestamp)
 
 
 // tag cloud -------------------------------------------------------------------
+function is_valid_tag(tag)
+{
+  return tag.match(/^[.a-zA-Z0-9]+$/) != null;
+}
+
+
+
+
 function format_tag_links(tags, joiner) 
 {
   if (joiner == undefined)
@@ -805,24 +990,39 @@ function format_tag_links(tags, joiner)
 
 
 
-function make_tag_cloud(data, max_usecount, show_hidden, exclude)
+function make_tag_cloud(data, show_hidden, exclude)
 {
+  var max_usecount = 0;
+  for (var i = 0; i < data.length; ++i)
+  {
+    var tag = data[i][0];
+    var usecount = data[i][1];
+
+    if (exclude != undefined && find_in_array(exclude, tag) != -1)
+      continue;
+
+    if (tag[0] == "." && !show_hidden)
+      continue;
+
+    if (usecount > max_usecount)
+      max_usecount = usecount;
+  }
+
   var html = '';
   for (var i = 0; i < data.length; ++i)
   {
     var tag = data[i][0];
 
-    if (exclude != undefined)
-      if (exclude.indexOf(tag) != -1)
-        continue;
+    if (exclude != undefined && find_in_array(exclude, tag) != -1)
+      continue;
 
     if (tag[0] == "." && !show_hidden)
       continue;
 
     var usecount = data[i][1];
     var usefraction = data[i][1]/max_usecount;
-    var sizefraction = 1-Math.pow(1-usefraction, 2);
-    var fontsize = Math.round(8.+usefraction*15);
+    var sizefraction = 1-Math.pow(1-usefraction, 0.8);
+    var fontsize = Math.round(7.+usefraction*15);
 
     html += ('<a class="taglink" style="font-size:[fs]pt">[tag]</a> '
       .allreplace("[fs]", fontsize)
@@ -835,20 +1035,58 @@ function make_tag_cloud(data, max_usecount, show_hidden, exclude)
 
 
 
-function update_tag_cloud()
+function update_tag_clouds()
 {
-  $.getJSON("/tags/get?withusecount", function (json)
+  update_main_tag_cloud();
+  update_subtag_cloud();
+}
+
+
+
+
+function update_main_tag_cloud()
+{
+  $.getJSON("/tags/get", function (json)
     {  
       $("#tagcloud").html(make_tag_cloud(
-          json.data, json.max_usecount,
-          $("#chk_tagcloud_show_hidden").get(0).checked));
-      make_tag_links($("#tagcloud a"));
+          json.tags, $("#chk_tagcloud_show_hidden").get(0).checked));
+      add_tag_behavior($("#tagcloud a"));
     });
 }
 
 
 
 
+function fill_subtag_cloud(data)
+{
+  $("#subtagcloud").html(
+    make_tag_cloud(data.tags, 
+      $("#chk_subtagcloud_show_hidden").get(0).checked, 
+      data.query_tags));
+  add_tag_behavior($("#subtagcloud a"));
+
+  $("#subtagcloud_search_tags").html(
+      format_tag_links(data.query_tags, " &middot; ")
+      );
+  add_tag_behavior($("#subtagcloud_search_tags a"));
+}
+
+
+
+
+function update_subtag_cloud()
+{
+  $.ajax({
+    dataType: 'json',
+    url: '/tags/get',
+    data: { query: $("#search").val() },
+    success: function(data, msg) { fill_subtag_cloud(data); }
+  });
+}
+
+
+
+
 function parse_tags(taglist_str)
 {
   var tags = taglist_str.split(",");
@@ -866,36 +1104,93 @@ function parse_tags(taglist_str)
 
 
 
-function click_tag(tag)
+function set_search(search, no_hist_update)
 {
-  var search = $("#search").val();
-  var tags = parse_tags(search);
-
-  var idx = tags.indexOf(tag);
-  if (idx == -1)
-    tags.push(tag);
-  else
-    tags.splice(idx, 1);
-
-  $("#search").val(tags.join(", "));
+  $("#search").val(search);
   $("#search").change();
+  if (!no_hist_update)
+    document.collection_manager.add_history_item();
 }
 
 
 
 
-function make_tag_links(jq_result)
+function click_tag(tag)
+{
+  var search = $("#search").val();
+
+  var idx = search.indexOf(tag);
+  if (idx == -1)
+  {
+    if (search.length == 0)
+      search = tag;
+    else
+      search = search + " " + tag;
+  }
+  else
+  {
+    search = search.replace(RegExp(re_escape("-"+tag+" "), "g"), "");
+    search = search.replace(RegExp(re_escape(" -"+tag), "g"), "");
+    search = search.replace(RegExp(re_escape(tag+" "), "g"), "");
+    search = search.replace(RegExp(re_escape(" "+tag), "g"), "");
+    search = search.replace(RegExp(re_escape(tag), "g"), "");
+  }
+
+  set_search(search);
+}
+
+
+
+
+function add_tag_behavior(jq_result)
 {
   jq_result.click(function()
     {
       click_tag($(this).html());
     });
   jq_result.dblclick(function()
-    {
-      var tag = $(this).html();
-      $("#search").val(tag);
-      $("#search").change();
-    });
+    { set_search($(this).html()); });
+  jq_result.contextMenu("tag_context_menu", {
+      bindings: {
+        'rename': function(tag_html) 
+        {
+          var old_name = $(tag_html).html();
+          var new_name = prompt(
+            "Rename tag '[old_name]' to:"
+            .allreplace("[old_name]", old_name),
+            old_name);
+          if (new_name != undefined && new_name != old_name)
+          {
+            if (!is_valid_tag(new_name))
+              alert("Cannot rename--not a valid tag.");
+            else
+            {
+              $.ajax({
+                type: 'POST',
+                dataType: 'text',
+                url: '/tags/rename',
+                data: {json: JSON.stringify({
+                  old_name: old_name,
+                  new_name: new_name
+                })},
+                error: function(req, stat, err) { report_error("Rename failed."); },
+                success: function(data, msg) { 
+                  set_message("Rename successful."); 
+                  update_tag_clouds();
+                  document.collection_manager.update(true);
+                }
+              });
+            }
+          }
+        },
+        'restrict': function(tag_html) {
+          click_tag($(tag_html).html());
+        },
+        'search': function(tag_html) {
+          set_search($(tag_html).html());
+        }
+      }
+      });
 }
 
 
@@ -904,20 +1199,33 @@ function make_tag_links(jq_result)
 // functions  ------------------------------------------------------------------
 $(document).ready(function()
 {
-  document.item_manager = new ItemCollectionManager();
-  document.item_manager.update();
+  var collection_manager = new ItemCollectionManager()
+  document.collection_manager = collection_manager;
 
   $("#navtabs > ul").tabs();
 
-  update_tag_cloud();
-  $("#chk_tagcloud_show_hidden").change(update_tag_cloud);
+  update_main_tag_cloud();
+  $("#chk_tagcloud_show_hidden").change(update_main_tag_cloud);
+  $("#chk_subtagcloud_show_hidden").change(update_subtag_cloud);
 
-  $("#btn_print").click(function()
-    {
-      var printurl = "/item/print_multi_by_tags?query="+escape($("#search").val());
-      if (self.view_time != null)
-        printurl += "&max_timestamp="+escape(self.view_time.toString());
-
-      window.open(printurl);
+  dhtmlHistory.initialize();
+  dhtmlHistory.addListener(
+    function (new_loc, data)
+    { 
+      collection_manager.handle_history_event(new_loc, data); 
     });
+  if (dhtmlHistory.isFirstLoad())
+    collection_manager.handle_history_event(dhtmlHistory.getCurrentLocation(), null);
+});
+
+
+window.dhtmlHistory.create({
+  toJSON: function(o) 
+  {
+    return JSON.stringify(o);
+  }, 
+  fromJSON: function(s) 
+  {
+    return JSON.parse(s);
+  }
 });
